//==========================================================================
// vec3f.hpp : 3d vector class declaration
//==========================================================================

#ifndef VEC3F
#define VEC3F

#include <stdio.h>
#include <math.h>

class Vec3f
{
  public:
    float x, y, z;

    Vec3f (void) 
      {};
    Vec3f (const float X, const float Y, const float Z) 
      { x=X; y=Y; z=Z; };
    Vec3f (const Vec3f& v) 
      { x=v.x; y=v.y; z=v.z; };
    Vec3f (float v[3])
      { x=v[0]; y=v[1]; z=v[2]; };
    void Set (const float X, const float Y, const float Z)
      { x=X; y=Y; z=Z; }
    void Set (float v[3])
      { x=v[0]; y=v[1]; z=v[2]; };

    operator float*()                             // FLOAT * CONVERSION
      { return (float *)&x; }
    operator const float*() const                 // CONST FLOAT * CONVERSION
      { return &x; }

    int operator == (const Vec3f& A)              // COMPARISON (==)
      { return (x==A.x && y==A.y && z==A.z); }

    Vec3f& operator = (const Vec3f& A)            // ASSIGNMENT (=)
      { x=A.x; y=A.y; z=A.z; 
        return(*this);  };
    Vec3f operator + (const Vec3f& A) const       // ADDITION (+)
      { Vec3f Sum(x+A.x, y+A.y, z+A.z); 
        return(Sum); };
    Vec3f operator - (const Vec3f& A) const       // SUBTRACTION (-)
      { Vec3f Diff(x-A.x, y-A.y, z-A.z);
        return(Diff); };
    float operator * (const Vec3f& A) const       // DOT-PRODUCT (*)
      { float DotProd = x*A.x+y*A.y+z*A.z; 
        return(DotProd); };
    Vec3f operator / (const Vec3f& A) const       // CROSS-PRODUCT (/)
      { Vec3f CrossProd(y*A.z-z*A.y, z*A.x-x*A.z, x*A.y-y*A.x);
        return(CrossProd); };
    Vec3f operator * (const float s) const        // MULTIPLY BY SCALAR V*s (*)
      { Vec3f Scaled(x*s, y*s, z*s); 
        return(Scaled); };
    Vec3f operator / (const float s) const        // DIVIDE BY SCALAR (/)
      { Vec3f Scaled(x/s, y/s, z/s);
        return(Scaled); };

    Vec3f CompMult (const Vec3f& A) const        // COMPONENT-WISE MULT
      { Vec3f Scaled(x*A.x, y*A.y, z*A.z);
        return(Scaled); };
    Vec3f CompDiv (const Vec3f& A) const         // COMPONENT-WISE DIVISION
      { Vec3f Scaled(x/A.x, y/A.y, z/A.z);
        return(Scaled); };

    friend inline Vec3f operator *(float s, const Vec3f& v)  // SCALAR MULT s*V
      { return Vec3f(v.x*s, v.y*s, v.z*s); }

    void operator += (const Vec3f A)              // ACCUMULATED VECTOR ADDITION (+=)
      { x+=A.x; y+=A.y; z+=A.z; };
    void operator -= (const Vec3f A)              // ACCUMULATED VECTOR SUBTRACTION (+=)
      { x-=A.x; y-=A.y; z-=A.z; };
    void operator *= (const float s)              // ACCUMULATED SCALAR MULT (*=)
      { x*=s; y*=s; z*=s; };
    void operator /= (const float s)              // ACCUMULATED SCALAR DIV (/=)
      { x/=s; y/=s; z/=s; };
    Vec3f operator - (void) const                 // NEGATION (-)
      { Vec3f Negated(-x, -y, -z);
        return(Negated); };

/*
    const float& operator [] (const int i) const         // ALLOWS VECTOR ACCESS AS AN ARRAY.
      { return( (i==0)?x:((i==1)?y:z) ); };
    float & operator [] (const int i)
      { return( (i==0)?x:((i==1)?y:z) ); };
*/

    float Length (void) const                     // LENGTH OF VECTOR
      { return ((float)sqrt(x*x+y*y+z*z)); };
    float LengthSqr (void) const                  // LENGTH OF VECTOR (SQUARED)
      { return (x*x+y*y+z*z); };
    void Normalize (void)                         // NORMALIZE VECTOR
      { float L = Length();                       // CALCULATE LENGTH
        if (L>0) { x/=L; y/=L; z/=L; } };         // DIV COMPONENTS BY LENGTH

    void UpdateMinMax(Vec3f &Min, Vec3f &Max)
    {
      if (x<Min.x) Min.x=x; else if (x>Max.x) Max.x=x;
      if (y<Min.y) Min.y=y; else if (y>Max.y) Max.y=y;
      if (z<Min.z) Min.z=z; else if (z>Max.z) Max.z=z;
    }

    void Print()
      { printf("(%.3f, %.3f, %.3f)\n",x, y, z); }

    static Vec3f ZERO;
};

#endif


