//============================================================================
// pgm.hpp : Portable Grayscale Map image format module
//============================================================================

#include <stdio.h>

//----------------------------------------------------------------------------
// READS AN IMAGE IN FROM A PPM FILE. RETURNS THE COLOR RGB ARRAY AND DIMENSIONS
// PERFORMS AUTO-ALLOCATION OF Color ARRAY IF SET TO NULL BEFORE CALLING; OTHERWISE
// ASSUMES THAT COLOR HAS BEEN PRE-ALLOCED.
//----------------------------------------------------------------------------
void LoadPGM(char *FileName, unsigned char* &GrayBuffer, int &Width, int &Height)
{
  FILE* fp = fopen(FileName, "rb");
  if (fp==NULL) 
    { printf("ERROR (LoadPGM): unable to open %s!\n",FileName);
      GrayBuffer=NULL; Width=0; Height=0; return; }
  int c,s;
  do{ do { s=fgetc(fp); } while (s!='\n'); } while ((c=fgetc(fp))=='#');
  ungetc(c,fp);
  fscanf(fp, "%d %d\n255\n", &Width, &Height);
  printf("Reading %dx%d PGM [%s]. . .\n", Width, Height, FileName);
  int NumComponents = Width*Height;
  if (GrayBuffer==NULL) GrayBuffer = new unsigned char[NumComponents];
  fread(GrayBuffer,NumComponents,1,fp);
  fclose(fp);
}

//----------------------------------------------------------------------------
// Writes an unsigned byte RGB color array out to a PPM file.
//----------------------------------------------------------------------------
void WritePGM(char *FileName, unsigned char* GrayBuffer, int Width, int Height)
{
  FILE* fp = fopen(FileName, "wb");
  if (fp==NULL) { printf("Error (WritePGM) : unable to open %s!\n",FileName); return; }
  printf("Writing %dx%d PGM [%s]. . .\n", Width, Height, FileName);
  fprintf(fp, "P5\n%d %d\n255\n", Width, Height);
  fwrite(GrayBuffer,1,Width*Height,fp);
  fclose(fp);
}
