//==========================================================================
// vec2f.hpp : 2d vector class declaration
//==========================================================================

#ifndef VEC2F
#define VEC2F

#include <stdio.h>
#include <math.h>

class Vec2f
{
  public:
    float x, y;

    Vec2f (void) {};
    Vec2f (const float X, const float Y) { x=X; y=Y; };
    Vec2f (const Vec2f& v) { x=v.x; y=v.y; };
    Vec2f (float v[2]) { x=v[0]; y=v[1]; };
    void Set (const float X, const float Y) { x=X; y=Y; }
    void Set (float v[2]) { x=v[0]; y=v[1]; };

    operator float*()                             // FLOAT * CONVERSION
      { return (float *)&x; }
    operator const float*() const                 // CONST FLOAT * CONVERSION
      { return &x; }

    int operator == (const Vec2f& A)              // COMPARISON (==)
      { return (x==A.x && y==A.y); }

    Vec2f& operator = (const Vec2f& A)            // ASSIGNMENT (=)
      { x=A.x; y=A.y;
        return(*this);  };
    Vec2f operator + (const Vec2f& A) const       // ADDITION (+)
      { Vec2f Sum(x+A.x, y+A.y); 
        return(Sum); };
    Vec2f operator - (const Vec2f& A) const       // SUBTRACTION (-)
      { Vec2f Diff(x-A.x, y-A.y);
        return(Diff); };
    float operator * (const Vec2f& A) const       // DOT-PRODUCT (*)
      { float DotProd = x*A.x+y*A.y; 
        return(DotProd); };
    float operator / (const Vec2f& A) const       // CROSS-PRODUCT (/)
      { float CrossProd = x*A.y-y*A.x;
        return(CrossProd); };
    Vec2f operator * (const float s) const        // MULTIPLY BY SCALAR (*)
      { Vec2f Scaled(x*s, y*s); 
        return(Scaled); };
    Vec2f operator / (const float s) const        // DIVIDE BY SCALAR (/)
      { Vec2f Scaled(x/s, y/s);
        return(Scaled); };

    friend inline Vec2f operator *(float s, const Vec2f& v)  // SCALAR MULT s*V
      { return Vec2f(v.x*s, v.y*s); }

    void operator += (const Vec2f A)              // ACCUMULATED VECTOR ADDITION (+=)
      { x+=A.x; y+=A.y; };
    void operator -= (const Vec2f A)              // ACCUMULATED VECTOR SUBTRACTION (+=)
      { x-=A.x; y-=A.y; };
    void operator *= (const float s)              // ACCUMULATED SCALAR MULT (*=)
      { x*=s; y*=s; };
    void operator /= (const float s)              // ACCUMULATED SCALAR DIV (/=)
      { x/=s; y/=s; };
    Vec2f operator - (void) const                 // NEGATION (-)
      { Vec2f Negated(-x, -y);
        return(Negated); };

/*
    const float& operator [] (const int i) const         // ALLOWS VECTOR ACCESS AS AN ARRAY.
      { return( (i==0)?x:y ); };
    float & operator [] (const int i)
      { return( (i==0)?x:y ); };
*/

    float Length (void) const                     // LENGTH OF VECTOR
      { return ((float)sqrt(x*x+y*y)); };
    float LengthSqr (void) const                  // LENGTH OF VECTOR (SQUARED)
      { return (x*x+y*y); };
    void Normalize (void)                         // NORMALIZE VECTOR
      { float L = Length();                       // CALCULATE LENGTH
        if (L>0) { x/=L; y/=L; } };               // DIV COMPONENTS BY LENGTH

    Vec2f Perpendicular()                         // RETURNS A PERPENDICULAR
      { Vec2f Perp(-y,x);
        return(Perp); }

    void Print()
      { printf("(%.3f, %.3f)\n",x, y); }
};

#endif


